@@ -1,6 +1,6 @@
 /* diritor.h: Iterator through entries in a directory.
  *
- * Copyright (C) 2007,2008 Olly Betts
+ * Copyright (C) 2007,2008,2010 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,12 +24,32 @@
 
 #include "safedirent.h"
 #include "safeerrno.h"
+#include "safefcntl.h"
 #include "safesysstat.h"
 
+#include <sys/types.h>
+#include <grp.h> // For getgrgid().
+#include <pwd.h> // For getpwuid().
+
+#ifdef HAVE_MAGIC_H
+#include <magic.h>
+#endif
+
 #include "common/noreturn.h"
 
+#include "loadfile.h"
+#include "runfilter.h" // For class ReadError.
+
 class DirectoryIterator {
+    static uid_t euid;
+
+#ifdef HAVE_MAGIC_H
+    static magic_t magic_cookie;
+#endif
+
     std::string path;
+    std::string::size_type path_len;
+
     DIR * dir;
     struct dirent *entry;
     struct stat statbuf;
@@ -38,6 +58,15 @@ class DirectoryIterator {
 
     void call_stat();
 
+    void ensure_statbuf_valid() {
+	if (!statbuf_valid) {
+	    call_stat();
+	    statbuf_valid = true;
+	}
+    }
+
+    void build_path();
+
   public:
 
     DirectoryIterator(bool follow_symlinks_)
@@ -58,6 +87,7 @@ class DirectoryIterator {
     //
     //  @return false if there are no more entries.
     bool next() {
+	path.resize(path_len);
 	errno = 0;
 	do {
 	    entry = readdir(dir);
@@ -71,6 +101,8 @@ class DirectoryIterator {
 
     const char * leafname() const { return entry->d_name; }
 
+    const std::string & pathname() const { return path; }
+
     typedef enum { REGULAR_FILE, DIRECTORY, OTHER } type;
 
     type get_type() {
@@ -96,7 +128,7 @@ class DirectoryIterator {
 	}
 #endif
 
-	if (!statbuf_valid) call_stat();
+	ensure_statbuf_valid();
 
 	if (S_ISREG(statbuf.st_mode)) return REGULAR_FILE;
 	if (S_ISDIR(statbuf.st_mode)) return DIRECTORY;
@@ -104,14 +136,66 @@ class DirectoryIterator {
     }
 
     off_t get_size() {
-	if (!statbuf_valid) call_stat();
+	ensure_statbuf_valid();
 	return statbuf.st_size;
     }
 
     off_t get_mtime() {
-	if (!statbuf_valid) call_stat();
+	ensure_statbuf_valid();
 	return statbuf.st_mtime;
     }
+
+    const char * get_owner() {
+	ensure_statbuf_valid();
+	struct passwd * pwentry = getpwuid(statbuf.st_uid);
+	return pwentry ? pwentry->pw_name : NULL;
+    }
+
+    const char * get_group() {
+	ensure_statbuf_valid();
+	struct group * grentry = getgrgid(statbuf.st_gid);
+	return grentry ? grentry->gr_name : NULL;
+    }
+
+    bool is_owner_readable() {
+	ensure_statbuf_valid();
+	return (statbuf.st_mode & S_IRUSR);
+    }
+
+    bool is_group_readable() {
+	ensure_statbuf_valid();
+	return (statbuf.st_mode & S_IRGRP);
+    }
+
+    bool is_other_readable() {
+	ensure_statbuf_valid();
+	return (statbuf.st_mode & S_IROTH);
+    }
+
+    bool try_noatime() {
+#if defined O_NOATIME && O_NOATIME != 0
+	if (euid == 0) return true;
+	ensure_statbuf_valid();
+	return statbuf.st_uid == euid;
+#else
+	return false;
+#endif
+    }
+
+#ifdef HAVE_MAGIC_H
+    std::string get_magic_mimetype();
+#else
+    std::string get_magic_mimetype() { return std::string(); }
+#endif
+
+    std::string file_to_string() {
+	build_path();
+	std::string out;
+	int flags = NOCACHE;
+	if (try_noatime()) flags |= NOATIME;
+	if (!load_file(path, out, flags)) throw ReadError();
+	return out;
+    }
 };
 
 #endif // OMEGA_INCLUDED_DIRITOR_H
