@@ -1,6 +1,6 @@
 /* metaxmlparse.h: subclass of HtmlParser for parsing OpenDocument's meta.xml.
  *
- * Copyright (C) 2006,2009 Olly Betts
+ * Copyright (C) 2006,2009,2010 Olly Betts
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,13 +23,13 @@
 #include "htmlparse.h"
 
 class MetaXmlParser : public HtmlParser {
-    enum { NONE, KEYWORDS, TITLE, SAMPLE } field;
+    enum { NONE, KEYWORDS, TITLE, SAMPLE, AUTHOR } field;
   public:
-    MetaXmlParser() { }
+    MetaXmlParser() : field(NONE) { }
     void process_text(const string &text);
     void opening_tag(const string &tag);
     void closing_tag(const string &tag);
-    string title, keywords, sample;
+    string title, keywords, sample, author;
 };
 
 #endif // OMEGA_INCLUDED_METAXMLPARSE_H
