@@ -34,7 +34,7 @@
 #include <cassert>
 #include <cctype>
 #include "safeerrno.h"
-#include <cstdio>
+#include <stdio.h>
 #include <cstdlib>
 #include <cstring>
 #include "strcasecmp.h"
@@ -45,11 +45,7 @@
 #include "safesysstat.h"
 #include "safefcntl.h"
 
-#ifdef HAVE_GETTIMEOFDAY
-#include <sys/time.h>
-#elif defined HAVE_FTIME
-#include <sys/timeb.h>
-#endif
+#include "realtime.h"
 
 #include <cdb.h>
 
@@ -63,6 +59,7 @@
 #include "str.h"
 #include "stringutils.h"
 #include "transform.h"
+#include "unixperm.h"
 #include "values.h"
 #include "weight.h"
 
@@ -109,6 +106,7 @@ static Xapian::Query query;
 Xapian::Query::op default_op = Xapian::Query::OP_OR; // default matching mode
 
 static Xapian::QueryParser qp;
+static Xapian::NumberValueRangeProcessor * size_vrp = NULL;
 static Xapian::Stem *stemmer = NULL;
 
 static string eval_file(const string &fmtfile);
@@ -122,9 +120,7 @@ static string queryterms;
 
 static string error_msg;
 
-static long int sec = 0, usec = -1;
-
-static bool suppress_http_headers = false;
+static double secs = -1;
 
 static const char DEFAULT_LOG_ENTRY[] =
 	"$or{$env{REMOTE_HOST},$env{REMOTE_ADDR},-}\t"
@@ -134,8 +130,6 @@ static const char DEFAULT_LOG_ENTRY[] =
 	"$query\t"
 	"$msize$if{$env{HTTP_REFERER},\t$env{HTTP_REFERER}}";
 
-static bool set_content_type = false;
-
 class MyStopper : public Xapian::Stopper {
   public:
     bool operator()(const string &t) const {
@@ -221,6 +215,11 @@ set_probabilistic(const string &oldp)
     qp.set_stopper(new MyStopper());
     qp.set_default_op(default_op);
     qp.set_database(db);
+    // FIXME: provide a custom VRP which handles size:10..20K, etc.
+    if (!size_vrp)
+	size_vrp = new Xapian::NumberValueRangeProcessor(VALUE_SIZE, "size:",
+							 true);
+    qp.add_valuerangeprocessor(size_vrp);
     // std::map::insert() won't overwrite an existing entry, so we'll prefer
     // the first user_prefix for which a particular term prefix is specified.
     map<string, string>::const_iterator pfx = option.lower_bound("prefix,");
@@ -400,31 +399,19 @@ run_query()
 	enquire->set_collapse_key(collapse_key);
     }
 
-#ifdef HAVE_GETTIMEOFDAY
-    struct timeval tv;
-    if (gettimeofday(&tv, 0) == 0) {
-	sec = tv.tv_sec;
-	usec = tv.tv_usec;
-    }
-#elif defined(FTIME_RETURNS_VOID)
-    struct timeb tp;
-    ftime(&tp);
-    sec = tp.time;
-    usec = tp.millitm * 1000;
-#elif defined(HAVE_FTIME)
-    struct timeb tp;
-    if (ftime(&tp) == 0) {
-	sec = tp.time;
-	usec = tp.millitm * 1000;
-    }
-#else
-    sec = time(NULL);
-    if (sec != (time_t)-1) usec = 0;
-#endif
     if (!query.empty()) {
+#if 0
+	// FIXME: If we start doing permissions checks based on $REMOTE_USER
+	// we're going to break some existing setups if users upgrade.  We
+	// probably want a way to set this from OmegaScript.
+	const char * remote_user = getenv("REMOTE_USER");
+	if (remote_user)
+	    apply_unix_permissions(query, remote_user);
+#endif
+
 	enquire->set_query(query);
 	// We could use the value of topdoc as first parameter, but we
-	// need to know the first few items on the mset to fake a
+	// need to know the first few items in the mset to fake a
 	// relevance set for topterms.
 	//
 	// If min_hits isn't set, check at least one extra result so we
@@ -434,45 +421,6 @@ run_query()
 				 topdoc + max(hits_per_page + 1, min_hits),
 				 &rset, mdecider);
     }
-    if (usec != -1) {
-#ifdef HAVE_GETTIMEOFDAY
-	if (gettimeofday(&tv, 0) == 0) {
-	    sec = tv.tv_sec - sec;
-	    usec = tv.tv_usec - usec;
-	    if (usec < 0) {
-		--sec;
-		usec += 1000000;
-	    }
-	} else {
-	    usec = -1;
-	}
-#elif defined(FTIME_RETURNS_VOID)
-	ftime(&tp);
-	sec = tp.time - sec;
-	usec = tp.millitm * 1000 - usec;
-	if (usec < 0) {
-	    --sec;
-	    usec += 1000000;
-	}
-#elif defined(HAVE_FTIME)
-	if (ftime(&tp) == 0) {
-	    sec = tp.time - sec;
-	    usec = tp.millitm * 1000 - usec;
-	    if (usec < 0) {
-		--sec;
-		usec += 1000000;
-	    }
-	} else {
-	    usec = -1;
-	}
-#else
-	usec = time(NULL);
-	if (usec != -1) {
-	    sec = sec - usec;
-	    usec = 0;
-	}
-#endif
-    }
 }
 
 static string
@@ -717,7 +665,62 @@ print_query_string(const char *after)
 }
 #endif
 
-static map<string, string> field;
+class Fields {
+    mutable Xapian::docid did_cached;
+    mutable map<string, string> fields;
+
+    void read_fields(Xapian::docid did) const;
+
+  public:
+    Fields() : did_cached(0) { }
+
+    const string & get_field(Xapian::docid did, const string & field) const {
+	if (did != did_cached) read_fields(did);
+	return fields[field];
+    }
+};
+
+void
+Fields::read_fields(Xapian::docid did) const
+{
+    fields.clear();
+    did_cached = did;
+    const string & data = db.get_document(did).get_data();
+
+    // Parse document data.
+    string::size_type i = 0;
+    const string & names = option["fieldnames"];
+    if (!names.empty()) {
+	// Each line is a field, with fieldnames taken from corresponding
+	// entries in the tab-separated list specified by $opt{fieldnames}.
+	string::size_type n = 0;
+	do {
+	    string::size_type n0 = n;
+	    n = names.find('\t', n);
+	    string::size_type i0 = i;
+	    i = data.find('\n', i);
+	    fields.insert(make_pair(names.substr(n0, n  - n0),
+				    data.substr(i0, i - i0)));
+	} while (++n && ++i);
+    } else {
+	// Each line is a field, in the format NAME=VALUE.  We assume the field
+	// name doesn't contain an "=".  Lines without an "=" are currently
+	// just ignored.
+	do {
+	    string::size_type i0 = i;
+	    i = data.find('\n', i);
+	    string line = data.substr(i0, i - i0);
+	    string::size_type j = line.find('=');
+	    if (j != string::npos) {
+		string & value = fields[line.substr(0, j)];
+		if (!value.empty()) value += '\t';
+		value += line.substr(j + 1);
+	    }
+	} while (++i);
+    }
+}
+
+static Fields fields;
 static Xapian::docid q0;
 static Xapian::doccount hit_no;
 static int percent;
@@ -742,6 +745,7 @@ CMD_def,
 CMD_defaultop,
 CMD_div,
 CMD_eq,
+CMD_emptydocs,
 CMD_env,
 CMD_error,
 CMD_field,
@@ -858,10 +862,11 @@ T(dbsize,	   0, 0, N, 0), // database size (# of documents)
 T(def,		   2, 2, 1, 0), // define a macro
 T(defaultop,	   0, 0, N, 0), // default operator: "and" or "or"
 T(div,		   2, 2, N, 0), // integer divide
+T(emptydocs,	   0, 1, N, 0), // list of empty documents
 T(env,		   1, 1, N, 0), // environment variable
 T(error,	   0, 0, N, 0), // error message
 T(eq,		   2, 2, N, 0), // test equality
-T(field,	   1, 1, N, 0), // lookup field in record
+T(field,	   1, 2, N, 0), // lookup field in record
 T(filesize,	   1, 1, N, 0), // pretty printed filesize
 T(filters,	   0, 0, N, 0), // serialisation of current filters
 T(filterterms,	   1, 1, N, 0), // list of terms with a given prefix
@@ -1190,6 +1195,18 @@ eval(const string &fmt, const vector<string> &param)
 	    case CMD_eq:
 		if (args[0] == args[1]) value = "true";
 		break;
+	    case CMD_emptydocs: {
+		string t;
+		if (!args.empty())
+		    t = args[0];
+		Xapian::PostingIterator i;
+		for (i = db.postlist_begin(t); i != db.postlist_end(t); ++i) {
+		    if (i.get_doclength() != 0) continue;
+		    if (!value.empty()) value += '\t';
+		    value += str(*i);
+		}
+		break;
+	    }
 	    case CMD_env: {
 		char *env = getenv(args[0].c_str());
 		if (env != NULL) value = env;
@@ -1201,9 +1218,12 @@ eval(const string &fmt, const vector<string> &param)
 		}
 		value = error_msg;
 		break;
-	    case CMD_field:
-		value = field[args[0]];
+	    case CMD_field: {
+		Xapian::docid did = q0;
+		if (args.size() > 1) did = string_to_int(args[1]);
+		value = fields.get_field(did, args[0]);
 		break;
+	    }
 	    case CMD_filesize: {
 		// FIXME: rounding?  i18n?
 		int size = string_to_int(args[0]);
@@ -1815,9 +1835,9 @@ eval(const string &fmt, const vector<string> &param)
 		value = str(topdoc / hits_per_page + 1);
 		break;
 	    case CMD_time:
-		if (usec != -1) {
+		if (secs >= 0) {
 		    char buf[64];
-		    my_snprintf(buf, sizeof(buf), "%ld.%06ld", sec, usec);
+		    my_snprintf(buf, sizeof(buf), "%.6f", secs);
 		    // MSVC's snprintf omits the zero byte if the string if
 		    // sizeof(buf) long.
 		    buf[sizeof(buf) - 1] = '\0';
@@ -2039,47 +2059,6 @@ print_caption(const string &fmt, const vector<string> &param)
     percent = mset.convert_to_percent(mset[hit_no]);
     collapsed = mset[hit_no].get_collapse_count();
 
-    Xapian::Document doc = db.get_document(q0);
-    string text = doc.get_data();
-
-    // Parse document data.
-    field.clear();
-    string::size_type i = 0;
-    string fieldnames = option["fieldnames"];
-    if (!fieldnames.empty()) {
-	// Each line is a field, with fieldnames taken from corresponding
-	// entries in the tab-separated list specified by $opt{fieldnames}.
-	string::size_type n = 0, n2;
-	while (true) {
-	    n2 = fieldnames.find('\t', n);
-	    string::size_type old_i = i;
-	    i = text.find('\n', i);
-	    field[fieldnames.substr(n, n2 - n)] = text.substr(old_i, i - old_i);
-	    if (n2 == string::npos || i == string::npos) break;
-	    ++i;
-	    n = n2 + 1;
-	}
-    } else {
-	// Each line is a field, in the format NAME=VALUE.  We assume the field
-	// name doesn't contain an "=".  Lines without an "=" are currently
-	// just ignored.
-	while (true) {
-	    string::size_type old_i = i;
-	    i = text.find('\n', i);
-	    string line = text.substr(old_i, i - old_i);
-	    string::size_type j = line.find('=');
-	    if (j != string::npos) {
-		string key = line.substr(0, j);
-		string value = field[key];
-		if (!value.empty()) value += '\t';
-		value += line.substr(j + 1);
-		field[key] = value;
-	    }
-	    if (i == string::npos) break;
-	    ++i;
-	}
-    }
-
     return eval(fmt, param);
 }
 
@@ -2221,7 +2200,11 @@ ensure_match()
 {
     if (done_query) return;
 
+    secs = RealTime::now();
     run_query();
+    if (secs != -1)
+	secs = RealTime::now() - secs;
+
     done_query = true;
     last = mset.get_matches_lower_bound();
     if (last == 0) {
